<?php
/**
 * @file
 * This file is part of miniOrange OTP Verification plugin.
 */

namespace Drupal\otp_verification;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Tests\field\Functional\reEnableModuleFieldTest;
use Drupal\otp_verification\MiniorangeOTPVerificationConstants;
use http\Encoding\Stream\Enbrotli;
use \Drupal\otp_verification\MiniorangeOTPVerificationCustomer;

/**
 * OTP Utilities Class.
 */
class MiniorangeOtpUtilities
{
  public static function AddSupportButton(array &$form, FormStateInterface $form_state)
  {
    $config = \Drupal::config('otp_verification.settings');

    $form['miniorange_saml_JS_for_visual_tour_and_support'] = array(
      '#attached' => array(
        'library' => array('otp_verification/otp_verification.support',
          'otp_verification/otp_verification.phone')
      ),
    );

    $form['miniorange_saml_idp_support_side_button'] = array(
      '#type' => 'button',
      '#value' => t('Support'),
      '#attributes' => array('style' => 'font-size: 15px;cursor: pointer;text-align: center;width: 150px;height: 35px;
                background: rgba(43, 141, 65, 0.93);color: #ffffff;border-radius: 3px;transform: rotate(90deg);text-shadow: none;
                position: relative;margin-left: -92px;top: 94px;'),
      '#prefix' => '<div id="mosaml-feedback-form" class="mo_saml_table_layout_support_btn">',
      '#suffix' => '<div id="Support_Section" class="mo_saml_table_layout_support_1">',
    );

    $form['markup_support_1'] = array(
      '#markup' => '<h3><b>Feature Request/Contact Us:</b></h3><div>Need any help? We can help you with configuring the module. Just send us a query and we will get back to you soon.<br /></div><br>',
    );

    $form['miniorange_otp_verification_email_address_support'] = array(
      '#type' => 'textfield',
      '#attributes' => array('placeholder' => 'Enter your Email'),
      '#default_value' => $config->get('miniorange_otp_verification_customer_admin_email'),
    );

    $form['miniorange_otp_verification_phone_number_support'] = array(
      '#type' => 'textfield',
      '#id' => 'query_phone',
      '#attributes' => array('class' => array('query_phone'),),
      '#default_value' => $config->get('miniorange_otp_verification_customer_admin_phone'),
    );

    $form['miniorange_otp_verification_support_query_support'] = array(
      '#type' => 'textarea',
      '#clos' => '10',
      '#rows' => '5',
      '#attributes' => array('placeholder' => 'Write your query here'),
    );

    $form['miniorange_saml_support_submit_click'] = array(
      '#type' => 'submit',
      '#value' => t('Submit Query'),
      '#submit' => array('\Drupal\otp_verification\MiniorangeOtpUtilities::send_support_query'),
      '#button_type' => 'primary',
    );

    $form['miniorange_saml_faq'] = array(
      '#markup' => '<a target="_blank" class="mbtn mo_otp-mbtn-primary mbtn-large" href="https://faq.miniorange.com/kb/otp-verification/"><u>FAQ</u></a>'

    );

    $form['miniorange_saml_support_note'] = array(
      '#markup' => '<div><br/>If you want custom features in the plugin, just drop an email to <a href="mailto:drupalsupport@xecurify.com">drupalsupport@xecurify.com</a></div>
                          </div></div><div hidden id="mosaml-feedback-overlay"></div>'
    );
  }

  /**
   * Get Support query data from support form
   */
  public static function send_support_query(&$form, $form_state)
  {
    $email = trim($form['miniorange_otp_verification_email_address_support']['#value']);
    $phone = trim($form['miniorange_otp_verification_phone_number_support']['#value']);
    $query = trim($form['miniorange_otp_verification_support_query_support']['#value']);

    self::send_query($email, $phone, $query);
  }


  public static function send_query($email, $phone, $query)
  {
    if (empty($email) || empty($query)) {
      \Drupal::messenger()->addMessage(t('The <b><u>Email</u></b> and <b><u>Query</u></b> fields are mandatory.'), 'error');
      return;
    } elseif (!\Drupal::service('email.validator')->isValid($email)) {
      \Drupal::messenger()->addMessage(t('The email address <b><i>' . $email . '</i></b> is not valid.'), 'error');
      return;
    }
    $support = new MiniorangeOTPVerificationSupport($email, $phone, $query);
    $support_response = $support->sendSupportQuery();
    if ($support_response) {
      \Drupal::messenger()->addMessage(t('Thanks for getting in touch! We will get back to you soon.'), 'status');
    } else {
      \Drupal::messenger()->addMessage(t('An error occured while sending support query.'), 'error');
    }
  }

  /**
   * Advertise 2FA
   */

  public static function Two_FA_Advertisement(array &$form, FormStateInterface $form_state)
  {
    global $base_url;
    $form['markup_idp_attr_hea555der_top_support'] = array(
      '#markup' => '<div class="mo_otp_verification_table_layout_support_1 mo_otp_verification_container" style="height:27em;">',
    );

    $form['miniorangerr_otp_email_address'] = array(
      '#markup' => '<div><h3 class="h_3"  >Looking for a Drupal Two-Factor Authentication(2FA)?<br></h3></div>
                        <div class="mo_adv_tfa"><img src="' . $base_url . '/' . drupal_get_path("module", "otp_verification") . '/includes/images/miniorange_i.png" alt="Simply Easy Learning" height="80px" width="80px" class="mo_img_adv"><h3 class="mo_txt_h3">Two-Factor Authentication (2FA)</h3></div>',

    );
    $form['minioranqege_otp_phone_number'] = array(
      '#markup' => '<div class="mo_paragraph"><p>Two Factor Authentication (2FA) module adds a second layer of authentication at the time of login to secure your Drupal accounts. It is a highly secure and easy to setup module which protects your site from hacks and unauthorized login attempts.</p></div>',
    );

    $form['miniorange_otp_2fa_button'] = array(
      '#markup' => '<div style="align:center;"> <a href="https://www.drupal.org/project/miniorange_2fa" class="mo_btn1 mbtn mo_otp-mbtn-success" target="_blank" id="tfa_btn_download">Download Module</a>
      <a href="https://plugins.miniorange.com/drupal-two-factor-authentication-2fa" class="mo_btn2 mbtn mo_otp-mbtn-primary" target="_blank" id="mo_tfa_btn_know">Know More</a>'

    );
  }

  /**
   * Check if curl is installed.
   */
  public static function isCurlInstalled()
  {
    if (in_array('curl', get_loaded_extensions())) {
      return 1;
    } else {
      return 0;
    }
  }

  /**
   * Check if customer is registered.
   */
  public static function isCustomerRegistered()
  {

    $config = \Drupal::config('otp_verification.settings');

    if ($config->get('miniorange_otp_verification_customer_admin_email') == NULL ||
      $config->get('miniorange_otp_verification_customer_id') == NULL ||
      $config->get('miniorange_otp_verification_customer_admin_token') == NULL ||
      $config->get('miniorange_otp_verification_customer_api_key') == NULL) {
      return FALSE;
    }
    return TRUE;
  }

  public static function sendToken($otp_options, $email, $phone)
  {
    $config = \Drupal::config('otp_verification.settings');
    $url = MiniorangeOTPVerificationConstants::BASE_URL . '/moas/api/auth/challenge';
    $customer_key = $config->get('miniorange_otp_verification_customer_id');
    $api_key = $config->get('miniorange_otp_verification_customer_api_key');

    if ($otp_options == 0) {
      $fields = array(
        'customerKey' => $customer_key,
        'email' => $email,
        'authType' => 'EMAIL',
        'transactionName' => 'Drupal OTP Verification',
      );
    }
    if ($otp_options == 1) {
      $fields = array(
        'customerKey' => $customer_key,
        'phone' => $phone,
        'authType' => 'SMS',
        'transactionName' => 'Drupal OTP Verification',
      );
    }

    $mo_otp_verification_customer = new MiniorangeOTPVerificationCustomer(null,null,null,null,$customer_key,$api_key);
    $content = $mo_otp_verification_customer->callService($url, $fields,true);

    $trans_status = json_decode($content)->status;
    $tx_id = json_decode($content)->txId;
    return array($trans_status, $tx_id);
  }

  public static function Is_Restricted_Domain($email_domain)
  {

    $config = \Drupal::config('otp_verification.settings');

    $enable_domain_restriction = $config->get('miniorange_block_domain_value');

    if ($enable_domain_restriction === FALSE)
      return FALSE;

    $domain = isset(explode('@', $email_domain)[1]) ? explode('@', $email_domain)[1] : '';

    if (is_null($domain) || empty($domain))
      return FALSE;

    $blockdomains = $config->get('miniorange_block_domains');
    $blockdomains = explode(';', $blockdomains);
    $white_or_black = $config->get('miniorange_domains_are_white_or_black');

    if ($white_or_black === 'white') {
      if (array_search($domain, $blockdomains) === FALSE) {
        return TRUE;
      } else return FALSE;
    } elseif ($white_or_black == 'black') {
      if (array_search($domain, $blockdomains) === FALSE) {
        return FALSE;
      } else return TRUE;
    }

  }

  public static function Is_Restricted_Country_Code($phone)
  {

    $config = \Drupal::config('otp_verification.settings');

    $enable_cc_restriction = $config->get('miniorange_enable_country_code_restriction');

    if ($enable_cc_restriction === 0)
      return FALSE;

    $country_codes = $config->get('miniorange_country_codes');
    $country_codes = explode(';', $country_codes);
    $allow_or_block = $config->get('miniorange_allow_or_block_country_code');

    if ($allow_or_block === 'allow') {

      foreach ($country_codes as $code) {
        if (strpos($phone, $code) !== false)
          return FALSE;
      }

      return TRUE;
    } elseif ($allow_or_block == 'block') {

      foreach ($country_codes as $code) {
        if (strpos($phone, $code) !== false)
          return TRUE;
      }

      return FALSE;
    }

  }

  public static function load_user_by_name($name)
  {
    $users = \Drupal::entityTypeManager()
      ->getStorage('user')
      ->loadByProperties([
        'name' => $name,
      ]);

    return $users ? reset($users) : FALSE;
  }

  public static function drupal_is_cli()
  {
    $server = \Drupal::request()->server;
    $server_software = $server->get('SERVER_SOFTWARE');
    $server_argc = $server->get('argc');
    return !isset($server_software) && (php_sapi_name() == 'cli' || (is_numeric($server_argc) && $server_argc > 0));
  }

  public static function mo_get_drupal_core_version() {
    return \DRUPAL::VERSION[0];
  }

  public static function isSessionStarted(){
    if (session_status() === PHP_SESSION_NONE) {
      session_start();
    }
  }

  public static function getNestedValue($arr){
    if( !is_array($arr) ){
      return $arr;
    }
    foreach ($arr as $key => $value){
      return self::getNestedValue($value);
    }
  }

  public static function set_otp_cookie($name, $value){
    setrawcookie('Drupal.visitor.' . $name, base64_encode(json_encode($value)), \Drupal::time()->getRequestTime() + 3900, '/');
  }

  public static function get_otp_cookie($name){
    return json_decode(base64_decode($_COOKIE['Drupal_visitor_'.$name]),true);
  }

}
